@@ -5,7 +5,7 @@ Stuff for processing Fortran, common to all fortran dialects.
 """
 
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# __COPYRIGHT__
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -28,7 +28,7 @@ Stuff for processing Fortran, common to all fortran dialects.
 #
 from __future__ import print_function
 
-__revision__ = "src/engine/SCons/Tool/FortranCommon.py 3a41ed6b288cee8d085373ad7fa02894e1903864 2019-01-23 17:30:35 bdeegan"
+__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
 import re
 import os.path
@@ -64,7 +64,8 @@ def _fortranEmitter(target, source, env):
     if not node.exists() and not node.is_derived():
        print("Could not locate " + str(node.name))
        return ([], [])
-    mod_regex = """(?i)^\s*MODULE\s+(?!PROCEDURE)(\w+)"""
+    # This has to match the def_regex in the Fortran scanner
+    mod_regex = r"""(?i)^\s*MODULE\s+(?!PROCEDURE|SUBROUTINE|FUNCTION|PURE|ELEMENTAL)(\w+)"""
     cre = re.compile(mod_regex,re.M)
     # Retrieve all USE'd module names
     modules = cre.findall(node.get_text_contents())
